/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.Date;

import org.ow2.bonita.definition.ActivityType;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.AutomaticBodyUUID;
import org.ow2.bonita.facade.uuid.RouteBodyUUID;
import org.ow2.bonita.facade.uuid.SubflowBodyUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EnvTool;


/**
 * @author Guillaume Porcher
 *
 */
public class ActivityBodyFactory {

  protected ActivityBodyFactory() {
  };

  public static ActivityBody getBody(final ActivityInstanceUUID activityUUID, final ActivityType activityType) {
    switch(activityType) {
      case route:
        return new RouteBodyImpl(new RouteBodyUUID(activityUUID.toString()));
      case subFlow:
        return new SubflowBodyImpl(new SubflowBodyUUID(activityUUID.toString()));
      case task:
        final TaskUUID taskUUID = new TaskUUID(activityUUID.toString());
        final TaskState state = TaskState.INITIAL;
        final Date createdDate = new Date();
        return new TaskFullInstanceImpl(taskUUID, state, createdDate, EnvTool.getUserId());
      case no:
        return new AutomaticBodyImpl(new AutomaticBodyUUID(activityUUID.toString()));
      case block:
      case tool:
      default:
        throw new BonitaRuntimeException("Ouch! Unsupported activity type: " + activityType);
    }
  }
}
