/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence;

import java.util.Set;

import org.jbpm.pvm.session.DbSession;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;


public interface QuerierDbSession extends DbSession {

  /**
   * TASK INSTANCE
   */
  Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState);
  Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(String userId, 
      ProcessInstanceUUID instanceUUID, TaskState taskState);
  ActivityFullInstance< TaskInstance > getTaskInstance(TaskUUID taskUUID);
  Set<ActivityFullInstance< TaskInstance >> getTaskInstances(ProcessInstanceUUID instanceUUID);
  
  /**
   * PROCESS INSTANCE
   */
  ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID);
  Set<ProcessFullInstance> getProcessInstances();
  Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processId);

  /**
   * ACTIVITY INSTANCE
   */
  ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId);
  Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance<ActivityBody> getActivityInstance(ActivityInstanceUUID activityInstanceUUID);

  /**
   * PACKAGE DEFINITION
   */
  Set<PackageFullDefinition> getPackages();
  Set<PackageFullDefinition> getPackages(String packageId);
  PackageFullDefinition getPackage(PackageDefinitionUUID packageUUID);
  PackageFullDefinition getPackage(String packageId, String version);
  Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState);
  Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState);
  PackageFullDefinition getLastDeployedPackage(String packageId, PackageDefinition.PackageState packageState);
  Set<PackageFullDefinition> getDependentPackages(PackageDefinitionUUID packageUUID);
  
  /**
   * PROCESS DEFINITION
   */
  Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageId);
  Set<ProcessFullDefinition> getProcesses();
  Set<ProcessFullDefinition> getProcesses(String processId);
  Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String packageId, String processId, ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, String version);
  ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID);
  ProcessFullDefinition getProcess(String packageId, String processId, String version);
  ProcessFullDefinition getLastDeployedProcess(String packageId, String processId, ProcessDefinition.ProcessState processState);
  ProcessFullDefinition getPackageProcess(PackageDefinitionUUID packageDefinitionUUID, String processId);
  
  
  /**
   * ACTIVITY DEFINITION
   */
  ActivityDefinition getActivityDefinition(ActivityDefinitionUUID activityDefinitionUUID);

}


