/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence.db;

import java.util.HashMap;
import java.util.Map;

import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlInstance;


public class DbRepositoryBuffer {

  private Map<ProcessInstanceUUID, XpdlInstance> instances = new HashMap<ProcessInstanceUUID, XpdlInstance>();
  private Map<ProcessInstanceUUID, XpdlInstance> removedInstances = new HashMap<ProcessInstanceUUID, XpdlInstance>();
  
  private Map<ProcessDefinitionUUID, XpdlProcess> processes = new HashMap<ProcessDefinitionUUID, XpdlProcess>();
  private Map<ProcessDefinitionUUID, XpdlProcess> removedProcesses = new HashMap<ProcessDefinitionUUID, XpdlProcess>();
  
  private Map<TaskUUID, TaskRunTime> tasks = new HashMap<TaskUUID, TaskRunTime>();
  private Map<TaskUUID, TaskRunTime> removedTasks = new HashMap<TaskUUID, TaskRunTime>();

  private Map<PackageDefinitionUUID, PackageClassData> packageClassDatas = 
    new HashMap<PackageDefinitionUUID, PackageClassData>();
  private Map<PackageDefinitionUUID, PackageClassData> removedPackageClassDatas = 
    new HashMap<PackageDefinitionUUID, PackageClassData>();
  
  private Map<String, GlobalClassData> globalClassDatas = 
    new HashMap<String, GlobalClassData>();
  private Map<String, GlobalClassData> removedGlobalClassDatas = 
    new HashMap<String, GlobalClassData>();

  public DbRepositoryBuffer() { }

  //All methods returning a Collection must never return null!
  //In that case, an empty collection must be returned
  

  /**
   * PROCESSES
   */
  public Map<ProcessDefinitionUUID, XpdlProcess> getProcesss() {
    return processes;
  }
  public XpdlProcess getProcess(ProcessDefinitionUUID processUUID) {
    return processes.get(processUUID);
  }

  public void addProcess(XpdlProcess process) {
    ProcessDefinitionUUID processUUID = process.getUUID();
    processes.put(processUUID, process);
    removedProcesses.remove(processUUID);
  }
  public void removeProcess(XpdlProcess process) {
    ProcessDefinitionUUID processUUID = process.getUUID();
    removedProcesses.put(processUUID, process);
    processes.remove(processUUID);
  }
  public boolean isProcessRemoved(ProcessDefinitionUUID processUUID) {
    return removedProcesses.containsKey(processUUID);
  }
  public boolean containsProcess(ProcessDefinitionUUID processUUID) {
    return processes.containsKey(processUUID);
  }
  
  /**
   * INSTANCES
   */
  public Map<ProcessInstanceUUID, XpdlInstance> getInstances() {
    return instances;
  }
  public XpdlInstance getInstance(ProcessInstanceUUID instanceUUID) {
    return instances.get(instanceUUID);
  }

  public void addInstance(XpdlInstance instance) {
    ProcessInstanceUUID instanceUUID = instance.getUUID();
    instances.put(instanceUUID, instance);
    removedInstances.remove(instanceUUID);
  }
  public void removeInstance(XpdlInstance instance) {
    ProcessInstanceUUID instanceUUID = instance.getUUID();
    removedInstances.put(instanceUUID, instance);
    instances.remove(instanceUUID);
  }
  public boolean isInstanceRemoved(ProcessInstanceUUID instanceUUID) {
    return removedInstances.containsKey(instanceUUID);
  }
  public boolean containsInstance(ProcessInstanceUUID instanceUUID) {
    return instances.containsKey(instanceUUID);
  }
  /**
   * TASKS
   */
  public Map<TaskUUID, TaskRunTime> getTasks() {
    return tasks;
  }
  public TaskRunTime getTask(TaskUUID taskUUID) {
    return tasks.get(taskUUID);
  }
  public void addTask(TaskRunTime task) {
    tasks.put(task.getUUID(), task);
    if (task.getXpdlExecution() != null) {
      instances.put(task.getInstanceUUID(), task.getXpdlExecution().getXpdlInstance());
    }
    removedTasks.remove(task.getUUID());
  }
  public void removeTask(TaskRunTime task) {
    TaskUUID taskUUID = task.getUUID();
    removedTasks.put(taskUUID, task);
    tasks.remove(taskUUID);
  }
  public boolean isTaskRemoved(TaskUUID taskUUID) {
    return removedTasks.containsKey(taskUUID);
  }
  public boolean containsTask(TaskUUID taskUUID) {
    return tasks.containsKey(taskUUID);
  }
  
  
  /**
   * PACKAGE CLASS DATA
   */
  public Map<PackageDefinitionUUID, PackageClassData> getPackageClassDatas() {
    return packageClassDatas;
  }
  public PackageClassData getPackageClassData(PackageDefinitionUUID packageUUID) {
    return packageClassDatas.get(packageUUID);
  }
  public void addPackageClassData(PackageClassData packageCD) {
    packageClassDatas.put(packageCD.getPackageUUID(), packageCD);
    removedPackageClassDatas.remove(packageCD.getPackageUUID());
  }
  public void removePackageClassData(PackageClassData packageCD) {
    PackageDefinitionUUID packageUUIDUUID = packageCD.getPackageUUID();
    removedPackageClassDatas.put(packageUUIDUUID, packageCD);
    packageClassDatas.remove(packageUUIDUUID);
  }
  public boolean isPackageClassDataRemoved(PackageDefinitionUUID packageUUID) {
    return removedPackageClassDatas.containsKey(packageUUID);
  }
  public boolean containsPackageClassData(PackageDefinitionUUID packageUUID) {
    return packageClassDatas.containsKey(packageUUID);
  }
  
  /**
   * GLOBAL CLASS DATA
   */
  public Map<String, GlobalClassData> getGlobalClassDatas() {
    return globalClassDatas;
  }
  public GlobalClassData getGlobalClassData(String className) {
    return globalClassDatas.get(className);
  }
  public void addGlobalClassData(GlobalClassData globalCD) {
    globalClassDatas.put(globalCD.getClassName(), globalCD);
    removedGlobalClassDatas.remove(globalCD.getClassName());
  }
  public void removeGlobalClassData(GlobalClassData globalCD) {
    String className = globalCD.getClassName();
    removedGlobalClassDatas.put(className, globalCD);
    globalClassDatas.remove(className);
  }
  public boolean isGlobalClassDataRemoved(String className) {
    return removedGlobalClassDatas.containsKey(className);
  }
  public boolean containsGlobalClassData(String className) {
    return globalClassDatas.containsKey(className);
  }
   
  
}


